import partial from 'lodash.partial';
import { mainHandler, createUserResolve } from '@resolve-js/runtime-base';
import { wrapApiHandler } from './wrap-api-handler';
export const handleApiGatewayEvent = async (lambdaEvent, lambdaContext, runtime, {
  performanceTracer,
  buildTimeConstants,
  routesTrie,
  domain,
  domainInterop,
  eventSubscriberScope,
  seedClientEnvs,
  eventListeners
}) => {
  const getCustomParameters = () => ({
    resolve: createUserResolve(runtime, {
      constants: buildTimeConstants,
      routesTrie,
      domain,
      domainInterop,
      eventSubscriberScope,
      performanceTracer,
      seedClientEnvs,
      eventListeners
    })
  });

  const executor = wrapApiHandler(mainHandler, partial(getCustomParameters, runtime), runtime.monitoring.group({
    Part: 'ApiHandler'
  }));
  const segment = performanceTracer.getSegment();
  const subSegment = segment.addNewSubsegment('apiHandler');

  try {
    return await executor(lambdaEvent, lambdaContext);
  } catch (error) {
    subSegment.addError(error);
    throw error;
  } finally {
    subSegment.close();
  }
};
//# sourceMappingURL=api-gateway-handler.js.map