import { getLog, pureRequire } from '@resolve-js/runtime-base';
import { errorBoundary } from 'resolve-cloud-common/utils/errorBoundary'; // eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore

import interopRequireDefault from '@babel/runtime/helpers/interopRequireDefault';
export const sendReactiveEvent = async event => {
  const {
    aggregateId,
    type
  } = event;
  const log = getLog(`sendReactiveEvent:${event.type}`);
  let escapeId;
  let escapeStr;
  let executeStatement;
  let invokeFunction;

  try {
    void ({
      escapeId
    } = interopRequireDefault(pureRequire('resolve-cloud-common/postgres')));
    void ({
      escapeStr
    } = interopRequireDefault(pureRequire('resolve-cloud-common/postgres')));
    void ({
      executeStatement
    } = interopRequireDefault(pureRequire('resolve-cloud-common/postgres')));
    void ({
      invokeFunction
    } = interopRequireDefault(pureRequire('resolve-cloud-common/lambda')));
  } catch {}

  const databaseNameAsId = escapeId(process.env.RESOLVE_EVENT_STORE_DATABASE_NAME);
  const subscriptionsTableNameAsId = escapeId(process.env.RESOLVE_SUBSCRIPTIONS_TABLE_NAME);
  const connectionIdsResult = await executeStatement({
    Region: process.env.AWS_REGION,
    ResourceArn: process.env.RESOLVE_EVENT_STORE_CLUSTER_ARN,
    SecretArn: process.env.RESOLVE_USER_SECRET_ARN,
    Sql: `SELECT "connectionId" FROM ${databaseNameAsId}.${subscriptionsTableNameAsId}
          WHERE (
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."eventTypes" #>
             ${escapeStr(`{${JSON.stringify(type)}}`)} =
             CAST('true' AS jsonb)
            OR
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."eventTypes" #> '{}' =
             CAST('null' AS jsonb) 
            ) AND (
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."aggregateIds" #>
             ${escapeStr(`{${JSON.stringify(aggregateId)}}`)} =
             CAST('true' AS jsonb)
            OR
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."aggregateIds" #> '{}' =
             CAST('null' AS jsonb) 
          )`
  });
  const errors = [];
  await Promise.all(connectionIdsResult.map(({
    connectionId
  }) => invokeFunction({
    Region: process.env.AWS_REGION,
    FunctionName: process.env.RESOLVE_WEBSOCKET_LAMBDA_ARN,
    Payload: {
      type: 'send',
      connectionId,
      data: {
        type: 'events',
        payload: {
          events: [event]
        }
      }
    }
  }).catch(errorBoundary(errors))));

  if (errors.length > 0) {
    log.warn(`Failed push event to websocket. ${errors}`);
  }
};
//# sourceMappingURL=send-reactive-event.js.map