"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleApiGatewayEvent = void 0;

var _lodash = _interopRequireDefault(require("lodash.partial"));

var _runtimeBase = require("@resolve-js/runtime-base");

var _wrapApiHandler = require("./wrap-api-handler");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const handleApiGatewayEvent = async (lambdaEvent, lambdaContext, runtime, {
  performanceTracer,
  buildTimeConstants,
  routesTrie,
  domain,
  domainInterop,
  eventSubscriberScope,
  seedClientEnvs,
  eventListeners
}) => {
  const getCustomParameters = () => ({
    resolve: (0, _runtimeBase.createUserResolve)(runtime, {
      constants: buildTimeConstants,
      routesTrie,
      domain,
      domainInterop,
      eventSubscriberScope,
      performanceTracer,
      seedClientEnvs,
      eventListeners
    })
  });

  const executor = (0, _wrapApiHandler.wrapApiHandler)(_runtimeBase.mainHandler, (0, _lodash.default)(getCustomParameters, runtime), runtime.monitoring.group({
    Part: 'ApiHandler'
  }));
  const segment = performanceTracer.getSegment();
  const subSegment = segment.addNewSubsegment('apiHandler');

  try {
    return await executor(lambdaEvent, lambdaContext);
  } catch (error) {
    subSegment.addError(error);
    throw error;
  } finally {
    subSegment.close();
  }
};

exports.handleApiGatewayEvent = handleApiGatewayEvent;
//# sourceMappingURL=api-gateway-handler.js.map