"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("source-map-support/register");

var _lodash = _interopRequireDefault(require("lodash.partial"));

var _core = require("@resolve-js/core");

var _runtimeBase = require("@resolve-js/runtime-base");

var _performanceTracerFactory = require("./performance-tracer-factory");

var _lambdaWorker = require("./lambda-worker");

var _uploaderFactory = require("./uploader-factory");

var _getReactiveSubscriptionFactory = require("./get-reactive-subscription-factory");

var _utils = require("./utils");

var _sendReactiveEvent = require("./send-reactive-event");

var _prepareAssemblies = require("./prepare-assemblies");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _runtimeBase.getLog)('aws-serverless-entry');

const entry = async (options, context) => {
  const {
    constants,
    domain,
    resolveVersion
  } = context;
  const assemblies = (0, _prepareAssemblies.prepareAssemblies)(context.assemblies, context);
  let subSegment = null;
  log.debug(`starting lambda 'cold start'`);
  const domainInterop = (0, _core.initDomain)(domain);

  try {
    log.debug('building lambda cold start context entries');
    const performanceTracer = await (0, _performanceTracerFactory.performanceTracerFactory)();
    const monitoring = (0, _runtimeBase.createCompositeMonitoringAdapter)(assemblies.monitoringAdapters);
    const segment = performanceTracer.getSegment();
    subSegment = segment.addNewSubsegment('initResolve');
    const uploader = await (0, _uploaderFactory.uploaderFactory)({
      uploaderAdapterFactory: assemblies.uploadAdapter
    });
    const getReactiveSubscription = (0, _getReactiveSubscriptionFactory.getReactiveSubscriptionFactory)();
    const coldStartContext = {
      seedClientEnvs: assemblies.seedClientEnvs,
      serverImports: assemblies.serverImports,
      domain,
      ...constants,
      assemblies,
      domainInterop,
      eventListeners: (0, _runtimeBase.gatherEventListeners)(domain, domainInterop),
      eventSubscriberScope: (0, _utils.getDeploymentId)(),
      upstream: true,
      resolveVersion,
      performanceTracer,
      monitoring,
      getReactiveSubscription,
      sendReactiveEvent: _sendReactiveEvent.sendReactiveEvent,
      routesTrie: (0, _runtimeBase.wrapTrie)(domain.apiHandlers, constants.staticRoutes, constants.rootPath),
      uploader,
      constants
    };
    log.debug(`lambda 'cold start' succeeded`);
    return (0, _lodash.default)(_lambdaWorker.lambdaWorker, coldStartContext);
  } catch (error) {
    log.error(`lambda 'cold start' failure`, error);
    if (subSegment != null) subSegment.addError(error);
  } finally {
    if (subSegment != null) {
      subSegment.close();
    }
  }

  return async () => {
    throw Error(`Lambda worker was not created due to cold start failure`);
  };
};

const factory = options => ({
  entry: (0, _lodash.default)(entry, options),
  execMode: 'external'
});

var _default = factory;
exports.default = _default;
//# sourceMappingURL=index.js.map