"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sendReactiveEvent = void 0;

var _runtimeBase = require("@resolve-js/runtime-base");

var _errorBoundary = require("resolve-cloud-common/utils/errorBoundary");

var _interopRequireDefault = _interopRequireDefault2(require("@babel/runtime/helpers/interopRequireDefault"));

function _interopRequireDefault2(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
const sendReactiveEvent = async event => {
  const {
    aggregateId,
    type
  } = event;
  const log = (0, _runtimeBase.getLog)(`sendReactiveEvent:${event.type}`);
  let escapeId;
  let escapeStr;
  let executeStatement;
  let invokeFunction;

  try {
    void ({
      escapeId
    } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
    void ({
      escapeStr
    } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
    void ({
      executeStatement
    } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
    void ({
      invokeFunction
    } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/lambda')));
  } catch {}

  const databaseNameAsId = escapeId(process.env.RESOLVE_EVENT_STORE_DATABASE_NAME);
  const subscriptionsTableNameAsId = escapeId(process.env.RESOLVE_SUBSCRIPTIONS_TABLE_NAME);
  const connectionIdsResult = await executeStatement({
    Region: process.env.AWS_REGION,
    ResourceArn: process.env.RESOLVE_EVENT_STORE_CLUSTER_ARN,
    SecretArn: process.env.RESOLVE_USER_SECRET_ARN,
    Sql: `SELECT "connectionId" FROM ${databaseNameAsId}.${subscriptionsTableNameAsId}
          WHERE (
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."eventTypes" #>
             ${escapeStr(`{${JSON.stringify(type)}}`)} =
             CAST('true' AS jsonb)
            OR
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."eventTypes" #> '{}' =
             CAST('null' AS jsonb) 
            ) AND (
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."aggregateIds" #>
             ${escapeStr(`{${JSON.stringify(aggregateId)}}`)} =
             CAST('true' AS jsonb)
            OR
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."aggregateIds" #> '{}' =
             CAST('null' AS jsonb) 
          )`
  });
  const errors = [];
  await Promise.all(connectionIdsResult.map(({
    connectionId
  }) => invokeFunction({
    Region: process.env.AWS_REGION,
    FunctionName: process.env.RESOLVE_WEBSOCKET_LAMBDA_ARN,
    Payload: {
      type: 'send',
      connectionId,
      data: {
        type: 'events',
        payload: {
          events: [event]
        }
      }
    }
  }).catch((0, _errorBoundary.errorBoundary)(errors))));

  if (errors.length > 0) {
    log.warn(`Failed push event to websocket. ${errors}`);
  }
};

exports.sendReactiveEvent = sendReactiveEvent;
//# sourceMappingURL=send-reactive-event.js.map