"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleWebsocketEvent = void 0;

var _jsonwebtoken = _interopRequireDefault2(require("jsonwebtoken"));

var _runtimeBase = require("@resolve-js/runtime-base");

var _interopRequireDefault = _interopRequireDefault2(require("@babel/runtime/helpers/interopRequireDefault"));

function _interopRequireDefault2(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
const log = (0, _runtimeBase.getLog)('resolve:runtime:websocket-event-handler'); //TODO: check in runtime that these environment variables are defined?

const region = process.env.AWS_REGION;
const websocketLambdaArn = process.env.RESOLVE_WEBSOCKET_LAMBDA_ARN;
const eventStoreClusterArn = process.env.RESOLVE_EVENT_STORE_CLUSTER_ARN;
const eventStoreDatabaseName = process.env.RESOLVE_EVENT_STORE_DATABASE_NAME;
const subscriptionsTableName = process.env.RESOLVE_SUBSCRIPTIONS_TABLE_NAME;
const userSecretArn = process.env.RESOLVE_USER_SECRET_ARN;
const encryptedDeploymentId = process.env.RESOLVE_ENCRYPTED_DEPLOYMENT_ID;
const LOAD_EVENTS_COUNT_LIMIT = 1000000; // 128kb is a limit for AWS WebSocket API message size

const LOAD_EVENTS_SIZE_LIMIT = 124 * 1024;
let escapeId;

try {
  void ({
    escapeId
  } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
} catch {}

const databaseNameAsId = escapeId(eventStoreDatabaseName);
const subscriptionsTableNameAsId = escapeId(subscriptionsTableName);

const handleWebsocketEvent = async ({
  method,
  payload
}, {
  eventStoreAdapter
}) => {
  log.debug(`dispatching lambda event to websocket`);

  switch (method) {
    case 'connect':
      {
        const {
          queryString,
          connectionId
        } = payload;
        const {
          token
        } = queryString;

        const {
          eventTypes,
          aggregateIds
        } = _jsonwebtoken.default.verify(token, encryptedDeploymentId);

        const eventTypeMap = eventTypes != null ? eventTypes.reduce((acc, type) => ({ ...acc,
          [type]: true
        }), {}) : null;
        const aggregateIdMap = aggregateIds != null ? aggregateIds.reduce((acc, id) => ({ ...acc,
          [id]: true
        }), {}) : null;
        let executeStatement;
        let escapeStr;

        try {
          void ({
            executeStatement
          } = (0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres'));
          void ({
            escapeStr
          } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
        } catch {}

        await executeStatement({
          Region: region,
          ResourceArn: eventStoreClusterArn,
          SecretArn: userSecretArn,
          Sql: `INSERT INTO ${databaseNameAsId}.${subscriptionsTableNameAsId} (
            "connectionId",
            "eventTypes",
            "aggregateIds"
          ) VALUES (
            ${escapeStr(connectionId)},
            ${escapeStr(JSON.stringify(eventTypeMap))},
            ${escapeStr(JSON.stringify(aggregateIdMap))}
          )`
        });
        break;
      }

    case 'receive':
      {
        const {
          connectionId,
          data
        } = payload;
        const {
          type,
          requestId,
          payload: messagePayload
        } = JSON.parse(data);

        switch (type) {
          case 'pullEvents':
            {
              let executeStatement;
              let escapeStr;

              try {
                void ({
                  executeStatement
                } = (0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres'));
                void ({
                  escapeStr
                } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
              } catch {}

              const connectionIdResult = await executeStatement({
                Region: region,
                ResourceArn: eventStoreClusterArn,
                SecretArn: userSecretArn,
                Sql: `SELECT * FROM ${databaseNameAsId}.${subscriptionsTableNameAsId}
              WHERE "connectionId" = ${escapeStr(connectionId)}`
              });

              if (connectionIdResult[0] != null) {
                const {
                  eventTypes,
                  aggregateIds
                } = connectionIdResult[0];
                const {
                  events,
                  cursor
                } = await eventStoreAdapter.loadEvents({
                  eventTypes: eventTypes === JSON.stringify(null) ? null : Object.keys(JSON.parse(eventTypes)),
                  aggregateIds: aggregateIds === JSON.stringify(null) ? null : Object.keys(JSON.parse(aggregateIds)),
                  limit: LOAD_EVENTS_COUNT_LIMIT,
                  eventsSizeLimit: LOAD_EVENTS_SIZE_LIMIT,
                  cursor: messagePayload.cursor
                });
                let invokeFunction;

                try {
                  void ({
                    invokeFunction
                  } = (0, _runtimeBase.pureRequire)('resolve-cloud-common/lambda'));
                } catch {}

                await invokeFunction({
                  Region: region,
                  FunctionName: websocketLambdaArn,
                  Payload: {
                    type: 'send',
                    connectionId,
                    data: {
                      type: 'pullEvents',
                      requestId,
                      payload: {
                        events,
                        cursor
                      }
                    }
                  }
                });
              }

              break;
            }

          default:
            {
              throw new Error(`The '${type}' message type is unknown`);
            }
        }

        break;
      }

    case 'disconnect':
      {
        const {
          connectionId
        } = payload;
        let executeStatement;
        let escapeStr;

        try {
          void ({
            executeStatement
          } = (0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres'));
          void ({
            escapeStr
          } = (0, _interopRequireDefault.default)((0, _runtimeBase.pureRequire)('resolve-cloud-common/postgres')));
        } catch {}

        await executeStatement({
          Region: region,
          ResourceArn: eventStoreClusterArn,
          SecretArn: userSecretArn,
          Sql: `DELETE FROM ${databaseNameAsId}.${subscriptionsTableNameAsId}
          WHERE "connectionId" = ${escapeStr(connectionId)}`
        });
        break;
      }

    default:
      throw new Error(`Wrong "${method}" websocket method`);
  }

  return {
    statusCode: 200
  };
};

exports.handleWebsocketEvent = handleWebsocketEvent;
//# sourceMappingURL=websocket-event-handler.js.map