import { lambdaWorker } from './lambda-worker';
import type { EventSubscriberNotification, EventSubscriberNotifier } from '@resolve-js/runtime-base';
import type { Trie } from 'route-trie';
import type { PerformanceTracer, Domain, Monitoring } from '@resolve-js/core';
import type { Assemblies, BuildTimeConstants, ReactiveSubscriptionFactory, Runtime, RuntimeFactoryParameters } from '@resolve-js/runtime-base';
export declare type OmitFirstParameter<F> = F extends (x: any, ...args: infer P) => infer R ? (...args: P) => R : never;
export declare type EventSubscriberInterface = {
    notifyEventSubscriber: EventSubscriberNotifier;
    invokeBuildAsync: (params: EventSubscriberNotification) => Promise<void>;
    getEventSubscriberDestination: (name: string) => string;
    ensureQueue: (name?: string) => Promise<void>;
    deleteQueue: (name?: string) => Promise<void>;
};
export declare type LambdaColdStartContext = {
    readonly performanceTracer: PerformanceTracer;
    readonly monitoring: Monitoring;
    readonly seedClientEnvs: RuntimeFactoryParameters['seedClientEnvs'];
    readonly serverImports: RuntimeFactoryParameters['serverImports'];
    readonly constants: BuildTimeConstants;
    readonly domain: RuntimeFactoryParameters['domain'];
    readonly domainInterop: Domain;
    readonly eventListeners: RuntimeFactoryParameters['eventListeners'];
    readonly eventSubscriberScope: string;
    readonly upstream: boolean;
    readonly resolveVersion: string;
    readonly routesTrie: Trie;
    readonly uploader: Runtime['uploader'];
    readonly sendReactiveEvent: RuntimeFactoryParameters['sendReactiveEvent'];
    readonly getReactiveSubscription: ReactiveSubscriptionFactory;
    readonly assemblies: Assemblies;
};
export declare type LambdaEventRecord = {
    eventSource: string | null;
    body: string;
};
export declare type LambdaContext = {
    invokedFunctionArn: string;
    functionName: string;
    getRemainingTimeInMillis: () => number;
    callbackWaitsForEmptyEventLoop: boolean;
    Records: Array<LambdaEventRecord | null>;
};
declare type LambdaEventBase = {
    Records?: LambdaEventRecord;
    httpMethod?: string;
    headers?: {
        [key: string]: string;
    };
    requestStartTime?: number;
};
export declare type BuildEventSubscriberLambdaEvent = {
    readonly resolveSource: 'BuildEventSubscriber';
    readonly eventSubscriber: string;
} & LambdaEventBase;
export declare type SchedulerLambdaEvent = {
    readonly resolveSource: 'Scheduler';
} & LambdaEventBase;
export declare type WebsocketLambdaEvent = {
    readonly resolveSource: 'Websocket';
} & LambdaEventBase;
export declare type CloudServiceLambdaEvent = {
    readonly resolveSource: 'DeployService';
    readonly part: 'bootstrap' | 'shutdown' | 'readModel' | 'saga';
    readonly operation: string;
    readonly soft: boolean;
} & LambdaEventBase;
export declare type ApiGatewayLambdaEvent = {
    readonly resolveSource: 'None';
    readonly path: string;
    readonly multiValueQueryStringParameters?: {
        [key: string]: string;
    };
    readonly body?: string | null;
} & LambdaEventBase;
export declare type LambdaEvent = BuildEventSubscriberLambdaEvent | SchedulerLambdaEvent | WebsocketLambdaEvent | CloudServiceLambdaEvent | ApiGatewayLambdaEvent;
export declare type RuntimeOptions = {};
export declare type WorkerArguments = Parameters<OmitFirstParameter<typeof lambdaWorker>>;
export declare type WorkerResult = {
    statusCode?: number;
    headers?: {
        [key: string]: string;
    };
    body?: string;
};
export {};
